
c========================================================================
      SUBROUTINE CALC_OUTER(count,this_rad, this_kp, this_en, this_lt,
     >             mean_e_all,std_e_all, five_e_all,
     >             tfive_e_all, med_e_all,sfive_e_all,nfive_e_all,
     >             final_e_all)
c========================================================================
c  calc.f
c  Calculate and return the model flux arrays for electron and ions

      IMPLICIT NONE

c     DEFINE INTEGER VARIABLES
      INTEGER(KIND=8) RADS, KPS, ENS, LTS, VALS
      INTEGER(KIND=8) IDX, IDY, e0_e, e1_e, lt0, lt1
      INTEGER(KIND=8) I, J, K, this_kp, r0, r1, count

c     DEFINE PARAMETERS
      PARAMETER(RADS=20, KPS=6, ENS=6, LTS=24, VALS=7)

c     DEFINE REAL ARRAYS
      REAL(KIND=8) mean_e_all(RADS,LTS,KPS,ENS)
      REAL(KIND=8) std_e_all(RADS,LTS,KPS,ENS)
      REAL(KIND=8) five_e_all(RADS,LTS,KPS,ENS)
      REAL(KIND=8) tfive_e_all(RADS,LTS,KPS,ENS)
      REAL(KIND=8) med_e_all(RADS,LTS,KPS,ENS)
      REAL(KIND=8) sfive_e_all(RADS,LTS,KPS,ENS)
      REAL(KIND=8) nfive_e_all(RADS,LTS,KPS,ENS)

      REAL(KIND=8) final_e_all(VALS)

      REAL(KIND=8) en_he(ENS),log_en_he(ENS)

c     DEFINE REAL VARIABLES
      REAL(KIND=8) this_rad,new_rad,this_en,this_lt,new_lt,log_this_en
      REAL(KIND=8) flux_11, flux_12, flux_21, flux_22
      REAL(KIND=8) x1, x2, y1, y2, P, v0, v1
      REAL(KIND=8) x, y, z0, z1, frac, dist

c    FILL AVERAGE CENTRE ENERGIES IN EV, NORMALISED BY SATELLITE
      DATA en_he/
     > 45000.D0, 59000.D0, 81000.D0, 110000.D0, 185000.D0, 325000.D0/

c     DEFINE REAL FUNCTIONS
      REAL(KIND=8) BILINEAR_OUTER

c     MATCH RAD TO FORTRAN ARRAY STARTING AT 1
      new_rad=this_rad+1.D0

c     FIND THE NEAREST NEIGHBOURS IN RADIUS: arrays go from 0 to 20
c     in steps of 1 R_E
      r0=FLOOR(new_rad)
      r1=CEILING(new_rad)

c     NOW INTERPOLATE BETWEEN THE TWO RADIUS VALUES
      dist=new_rad
      frac=dist-r0

c     CONVERT ENERGY BINS TO LOG10
      DO 10 I=1,ENS
        log_en_he(I)=DLOG10(en_he(I))
 10   CONTINUE 

c     CONVERT CHOSEN ENERGY TO LOG10
      log_this_en=DLOG10(this_en)

c     =====================ELECTRONS===================================
c
c     FIND THE NEAREST NEIGHBOURS IN ENERGY (ELECTRONS)
      IDX=1
c      PRINT*, this_en
      IF (this_en.LT.en_he(ENS)) THEN
 20     IF (log_en_he(IDX).LE.log_this_en) THEN 
c          PRINT*, log_en_he(IDX), log_this_en
          IDX=IDX+1
          GOTO 20
        ENDIF
        e0_e=IDX-1
        e1_e=IDX
      ENDIF
      IF (this_en.GT.en_he(ENS)) THEN
        e0_e=ENS-1
        e1_e=ENS
      ENDIF
      IF (this_en.EQ.en_he(ENS)) THEN
        e0_e=ENS-1
        e1_e=ENS
      ENDIF

c     MATCH LTS TO FORTRAN ARRAY STARTING AT 1
      new_lt=this_lt+1

c     BRACKET IN LOCAL TIME
c     DATA ARRAY WRITTEN FROM 0 to 23 LT; ADJUST to 1 to 24
      lt0=FLOOR(new_lt)
      lt1=lt0+1.D0
      IF (lt1.GT.24.D0) lt1=lt1-24.D0

c     SET LTs TO ACTUAL VALS AND CONVERT ENERGIES TO LOGS
      x1=lt0
      x2=lt1
      y1=log_en_he(e0_e)
      y2=log_en_he(e1_e)
      x=new_lt
      y=log_this_en
c
c     GET FOUR NEAREST NEIGHBOUR FLUX VALUES TO THE CHOSEN POINT (E)
c     Mean

c      print*, r0, lt0, this_kp, e0_e
c      STOP

      flux_11=mean_e_all(r0, lt0, this_kp, e0_e)
      flux_12=mean_e_all(r0, lt0, this_kp, e1_e)
      flux_21=mean_e_all(r0, lt1, this_kp, e0_e)
      flux_22=mean_e_all(r0, lt1, this_kp, e1_e)

      z0=BILINEAR_OUTER(x1,x2,y1,y2,x,y,flux_11,flux_12,flux_21,flux_22)

      flux_11=mean_e_all(r1, lt0, this_kp, e0_e)
      flux_12=mean_e_all(r1, lt0, this_kp, e1_e)
      flux_21=mean_e_all(r1, lt1, this_kp, e0_e)
      flux_22=mean_e_all(r1, lt1, this_kp, e1_e)

c      PRINT*,x1,x2,y1,y2,x,y,flux_11,flux_12,flux_21,flux_22
      z1=BILINEAR_OUTER(x1,x2,y1,y2,x,y,flux_11,flux_12,flux_21,flux_22)
      final_e_all(1)=((z1-z0)*frac)+z0

c     GET FOUR NEAREST NEIGHBOUR FLUX VALUES TO THE CHOSEN POINT (E)
c     Standard Deviation
      flux_11=std_e_all(r0, lt0, this_kp, e0_e)
      flux_12=std_e_all(r0, lt0, this_kp, e1_e)
      flux_21=std_e_all(r0, lt1, this_kp, e0_e)
      flux_22=std_e_all(r0, lt1, this_kp, e1_e)
      z0=BILINEAR_OUTER(x1,x2,y1,y2,x,y,flux_11,flux_12,flux_21,flux_22)

      flux_11=std_e_all(r1, lt0, this_kp, e0_e)
      flux_12=std_e_all(r1, lt0, this_kp, e1_e)
      flux_21=std_e_all(r1, lt1, this_kp, e0_e)
      flux_22=std_e_all(r1, lt1, this_kp, e1_e)
      z1=BILINEAR_OUTER(x1,x2,y1,y2,x,y,flux_11,flux_12,flux_21,flux_22)

      final_e_all(2)=((z1-z0)*frac)+z0
c      print*, 'here2', z0, z1, final_e_all(2)

c     GET FOUR NEAREST NEIGHBOUR FLUX VALUES TO THE CHOSEN POINT (E)
c     5th Percentile
      flux_11=five_e_all(r0, lt0, this_kp, e0_e)
      flux_12=five_e_all(r0, lt0, this_kp, e1_e)
      flux_21=five_e_all(r0, lt1, this_kp, e0_e)
      flux_22=five_e_all(r0, lt1, this_kp, e1_e)
      z0=BILINEAR_OUTER(x1,x2,y1,y2,x,y,flux_11,flux_12,flux_21,flux_22)

      flux_11=five_e_all(r1, lt0, this_kp, e0_e)
      flux_12=five_e_all(r1, lt0, this_kp, e1_e)
      flux_21=five_e_all(r1, lt1, this_kp, e0_e)
      flux_22=five_e_all(r1, lt1, this_kp, e1_e)
      z1=BILINEAR_OUTER(x1,x2,y1,y2,x,y,flux_11,flux_12,flux_21,flux_22)

      final_e_all(3)=((z1-z0)*frac)+z0
c      print*, 'here3', z0, z1, final_e_all(3)

c     GET FOUR NEAREST NEIGHBOUR FLUX VALUES TO THE CHOSEN POINT (E)
c     25th Percentile
      flux_11=tfive_e_all(r0, lt0, this_kp, e0_e)
      flux_12=tfive_e_all(r0, lt0, this_kp, e1_e)
      flux_21=tfive_e_all(r0, lt1, this_kp, e0_e)
      flux_22=tfive_e_all(r0, lt1, this_kp, e1_e)
      z0=BILINEAR_OUTER(x1,x2,y1,y2,x,y,flux_11,flux_12,flux_21,flux_22)

      flux_11=tfive_e_all(r1, lt0, this_kp, e0_e)
      flux_12=tfive_e_all(r1, lt0, this_kp, e1_e)
      flux_21=tfive_e_all(r1, lt1, this_kp, e0_e)
      flux_22=tfive_e_all(r1, lt1, this_kp, e1_e)
      z1=BILINEAR_OUTER(x1,x2,y1,y2,x,y,flux_11,flux_12,flux_21,flux_22)

      final_e_all(4)=((z1-z0)*frac)+z0
c      print*, 'here4', z0, z1, final_e_all(4)

c     GET FOUR NEAREST NEIGHBOUR FLUX VALUES TO THE CHOSEN POINT (E)
c     Median
      flux_11=med_e_all(r0, lt0, this_kp, e0_e)
      flux_12=med_e_all(r0, lt0, this_kp, e1_e)
      flux_21=med_e_all(r0, lt1, this_kp, e0_e)
      flux_22=med_e_all(r0, lt1, this_kp, e1_e)
      z0=BILINEAR_OUTER(x1,x2,y1,y2,x,y,flux_11,flux_12,flux_21,flux_22)

      flux_11=med_e_all(r1, lt0, this_kp, e0_e)
      flux_12=med_e_all(r1, lt0, this_kp, e1_e)
      flux_21=med_e_all(r1, lt1, this_kp, e0_e)
      flux_22=med_e_all(r1, lt1, this_kp, e1_e)
      z1=BILINEAR_OUTER(x1,x2,y1,y2,x,y,flux_11,flux_12,flux_21,flux_22)

      final_e_all(5)=((z1-z0)*frac)+z0
c      print*, 'here5', z0, z1, final_e_all(5)

c     GET FOUR NEAREST NEIGHBOUR FLUX VALUES TO THE CHOSEN POINT (E)
c     75th Percentile
      flux_11=sfive_e_all(r0, lt0, this_kp, e0_e)
      flux_12=sfive_e_all(r0, lt0, this_kp, e1_e)
      flux_21=sfive_e_all(r0, lt1, this_kp, e0_e)
      flux_22=sfive_e_all(r0, lt1, this_kp, e1_e)
      z0=BILINEAR_OUTER(x1,x2,y1,y2,x,y,flux_11,flux_12,flux_21,flux_22)

      flux_11=sfive_e_all(r1, lt0, this_kp, e0_e)
      flux_12=sfive_e_all(r1, lt0, this_kp, e1_e)
      flux_21=sfive_e_all(r1, lt1, this_kp, e0_e)
      flux_22=sfive_e_all(r1, lt1, this_kp, e1_e)
      z1=BILINEAR_OUTER(x1,x2,y1,y2,x,y,flux_11,flux_12,flux_21,flux_22)

      final_e_all(6)=((z1-z0)*frac)+z0
c      print*, 'here6', z0, z1, final_e_all(6)

c     GET FOUR NEAREST NEIGHBOUR FLUX VALUES TO THE CHOSEN POINT (E)
c     95th Percentile
      flux_11=nfive_e_all(r0, lt0, this_kp, e0_e)
      flux_12=nfive_e_all(r0, lt0, this_kp, e1_e)
      flux_21=nfive_e_all(r0, lt1, this_kp, e0_e)
      flux_22=nfive_e_all(r0, lt1, this_kp, e1_e)
      z0=BILINEAR_OUTER(x1,x2,y1,y2,x,y,flux_11,flux_12,flux_21,flux_22)

      flux_11=nfive_e_all(r1, lt0, this_kp, e0_e)
      flux_12=nfive_e_all(r1, lt0, this_kp, e1_e)
      flux_21=nfive_e_all(r1, lt1, this_kp, e0_e)
      flux_22=nfive_e_all(r1, lt1, this_kp, e1_e)
      z1=BILINEAR_OUTER(x1,x2,y1,y2,x,y,flux_11,flux_12,flux_21,flux_22)

      final_e_all(7)=((z1-z0)*frac)+z0
c      print*, 'here7', z0, z1, final_e_all(7)

c      WRITE(6,*) '--- CALCULATIONS COMPLETE ---'

c      DO 80 I=1,24 
c        WRITE(6,*) mean_e_all(r0,I,this_kp, e0_e)
c 80   CONTINUE
c      STOP

      RETURN

      END
c===================END SUBROUTINE CALC==================================
